import { Http, IEventParam, Metadata } from '@zeedhi/core';
import {
  AlertService, Form, Modal, ModalService, IGridEvent,
} from '@zeedhi/common';
import { ZdGrid } from '@zeedhi/vuetify';

export class Cmp99000GeracaoAfPrevisaoController {
  public FiltroModal!: Modal;
  
  public dateValue: string | string[] = '';
  
  public filterValueCDFILIOPELOG: any = null;
  
  public filterForm: any = [];
  
  public filters: any = [];
  
  public cdfilialArray: any = [];
  
  public cdFornArray: any = [];
  
  public paraSupri: any = [];
  
  public dataGrid: any = [];
  
  public currentTab: any = 'G';
  
  public lastTabGetted: any = '';
  
  public cdOperador: any = '';
  
  public nrapurprec: any = '';
  
  public async createFiltroModal() {
    this.FiltroModal = await ModalService.createFromJson('pages/cmp99000/cmp99000_filtro', true);
 
    this.FiltroModal.show();
  }
  
  public onFilterOpen() {
    this.getParaSupri();
  }

  public selectChange({ component }: IEventParam<any>) {
    const nrapurprec = Metadata.getInstance<any>('NRAPURPREC');
    
    if (component.name === 'CDFILIAL') {
      nrapurprec.datasource.filter.CDFILIAL = 'T';
    
      if (component.selectValue != null) {
        this.cdfilialArray = this.mountArrayFilter(component.selectValue, 'CDFILIAL');
        nrapurprec.datasource.filter.CDFILIAL = this.cdfilialArray;
        nrapurprec.disabled = false;
      }
      
      nrapurprec.datasource.get();
    }
    
    if (component.name === 'CDFORNERELACAF') {
      const fornecedor = Metadata.getInstance<any>('CDFORNECED');
      
      fornecedor.datasource.filter.P_CDFORNERELACAF = 'T';
      
      if (component.selectValue != null) {
        fornecedor.datasource.filter.P_CDFORNERELACAF = component.selectValue.CDFORNECED;
      }
      
      fornecedor.datasource.get();
    }
    
    if (component.name === 'CDFORNECED') {
      nrapurprec.datasource.filter.CDFORNECED = 'T';
      
      if (component.selectValue != null) {
        this.cdFornArray = this.mountArrayFilter(component.selectValue, 'CDFORNECED');
        nrapurprec.datasource.filter.CDFORNECED = this.cdFornArray;
      }
    
      nrapurprec.disabled = false;
      
      nrapurprec.datasource.get();
    }
    
    if (component.name === 'CDCOMPRADOR') {
      nrapurprec.disabled = true;
      if (component.selectValue != null) {
        nrapurprec.datasource.filter.CDOPERADOR = component.selectValue.CDOPERADOR;
        nrapurprec.disabled = false;
        nrapurprec.datasource.get();
      }
    }
    
    if (component.name === 'NRAPURPREC') {
      Metadata.getInstance<any>('PERIODO').addValidation('required');
      Metadata.getInstance<any>('DTENTREGA').addValidation('required');

      if (nrapurprec.selectValue !== null) {
        Metadata.getInstance<any>('PERIODO').clearValidations();
        Metadata.getInstance<any>('DTENTREGA').clearValidations();
      }
    }
  }
  
  public destroyFiltroModal() {
    this.FiltroModal?.destroy();
  }
  
  public fecharFiltroModal() {
    this.FiltroModal.hide();
  }
 
  public async abrirFiltroModal({ event }: IGridEvent) {
    event?.preventDefault();
    
    this.FiltroModal.show();
  }
  
  public async validaFiltro() {
    const form = Metadata.getInstance<Form>('formFiltro');
    const valid = form.validate();

    if (!valid) {
      AlertService.show({
        top: true,
        right: true,
        timeout: 2000,
        name: 'filtroInvalidoAltert',
        text: 'Preencha todos os campos obrigatórios!',
        color: 'error',
      });
      
      return;
    }
    
    form.submit();
    this.filterForm = form;
    this.loadGrid();
  }
  
  public getFilters() {
    const filterForm = this.filterForm.internalValue;

    const params = {
      CDFORNERELACAF: filterForm.CDFORNERELACAF != null ? filterForm.CDFORNERELACAF.CDFORNECED : 'T',
      CDFORNECED: filterForm.CDFORNECED.length > 0 ? this.mountArrayFilter(filterForm.CDFORNECED, 'CDFORNECED') : 'T',
      CDFILIAL: this.mountArrayFilter(filterForm.CDFILIAL, 'CDFILIAL'),
      CDCOMPRADOR: filterForm.CDCOMPRADOR.CDOPERADOR,
      TIPOPREVISAO: filterForm.TIPOPREVISAO.id,
      CDSUBGRUPPROD: filterForm.CDSUBGRUPPROD.length > 0 ? this.mountArrayFilter(filterForm.CDSUBGRUPPROD, 'CDSUBGRPROD') : 'T',
      CDPRODINI: filterForm.CDPRODINI.CDARVPROD,
      CDPRODFIN: filterForm.CDPRODFIN.CDARVPROD,
      CDMARCA: filterForm.CDMARCA.length > 0 ? this.mountArrayFilter(filterForm.CDMARCA, 'CDMARCA') : 'T',
      DTINICIAL: filterForm.PERIODO[0],
      DTFINAL: filterForm.PERIODO[1],
      QTDIAESTOQUE: filterForm.QTDIAESTOQUE != null ? filterForm.QTDIAESTOQUE.ROW_NUM : null,
      DTENTREGA: filterForm.DTENTREGA,
      CDTIPOCUSTO: filterForm.CDTIPOCUSTO != null ? filterForm.CDTIPOCUSTO.CDTIPOCUSTO : '',
      NRAPURPREC: filterForm.NRAPURPREC != null ? filterForm.NRAPURPREC.NRAPURPREC : this.nrapurprec,
      CDFILCENTCMP: this.paraSupri != null ? this.paraSupri.CDFILCENTCMP : '',
      NRORG: this.paraSupri != null ? this.paraSupri.NRORG : '',
      TIPOTELA: this.currentTab,
    };
    
    return params;
  }
  
  public mountArrayFilter(items: any, fieldName: any) {
    const newArray: Array<any> = [];

    if (items) {
      items.forEach((item: any) => {
        newArray.push(item[fieldName]);
      });
    }
    
    if (newArray.toString() !== 'underfined' && newArray.toString() !== null && newArray.toString() !== '') {
      const concactedArray = newArray.toString().replaceAll(',', '_');
      
      return String('_').concat(concactedArray.concat(String('_')));
    }
    
    return null;
  }
  
  public loadGrid() {
    this.filters = this.getFilters();
   
    Http.post('/getGeracaoAfPrevisao', this.filters).then((response) => {
      const grid = Metadata.getInstance<ZdGrid>('cmp99000Grid');
         
      if (response.data.error) {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'noData',
          text: response.data.error,
          color: 'error',
        });
      }
      
      this.dataGrid = response.data.dataset.data;

      if (this.currentTab === 'G') {
        grid.datasource.data = response.data.dataset.data;
      }
     
      if (this.currentTab === 'C') {
        this.newCrossTab(response.data.dataset.data);
      }
      
      const nrapurprec = Metadata.getInstance<any>('NRAPURPREC');

      if (nrapurprec.selectValue === null) {
        this.nrapurprec = response.data.dataset.data[0].NRAPURPREC;
      }
    });
    
    this.fecharFiltroModal();
  }
  
  public newCrossTab(dataGrid: any) {
    const crossTab = Metadata.getInstance<any>('cmp99000Crosstab');

    crossTab.datasource.data = dataGrid;
  }
  
  public async getParaSupri() {
    const parasupri = Metadata.getInstance<any>('PARASUPRI');
    const paraSupriData = await parasupri.datasource.get();

    this.paraSupri = paraSupriData[0];

    this.useParaSupri();
  }

  public useParaSupri() {
    if (typeof this.paraSupri !== 'undefined' && typeof this.paraSupri !== null) {
      const subgroup = Metadata.getInstance<any>('CDSUBGRUPPROD');
      const nrapurprec = Metadata.getInstance<any>('NRAPURPREC');
        
      subgroup.datasource.filter.P_CDGRUPPROD = this.paraSupri.CDGRUPPROD;
      nrapurprec.datasource.filter.CDFILCENTCMP = this.paraSupri.CDFILCENTCMP;
       
      subgroup.datasource.get().then((response: any) => {
        subgroup.updateData(response);
      });
    }
    
    this.fillingTheFilters();
  }
  
  public async fillingTheFilters() {
    const cdfilial = Metadata.getInstance<any>('CDFILIAL');
    const cdcomprador = Metadata.getInstance<any>('CDCOMPRADOR');
    const cdmarca = Metadata.getInstance<any>('CDMARCA');
    const cdtipocusto = Metadata.getInstance<any>('CDTIPOCUSTO');
    
    Promise.all([
      cdfilial.datasource.get(),
      cdcomprador.datasource.get(),
      cdmarca.datasource.get(),
      cdtipocusto.datasource.get(),
    ]);
    
    Http.get('/getSessionInfos').then((response) => {
      const sessionInfos = response.data.dataset.data;
      
      this.cdOperador = sessionInfos.CDOPERADOR;
      this.fillingField(cdfilial, 'CDFILIAL', sessionInfos.CDFILIAL);
      this.fillingField(cdcomprador, 'CDOPERADOR', sessionInfos.CDOPERADOR);
      this.fillingField(cdmarca, 'NMMARCA', 'PADRAO');
      this.fillingField(cdtipocusto, 'CDTIPOCUSTO', '01');
    });
  }
  
  public fillingField(field: any, keyName: any, sessionValue: any) {
    if (field.datasource.data.length > 0) {
      field.datasource.data.forEach((item: any) => {
        if (item[keyName] === sessionValue) {
          if (field.component === 'ZdSelect') {
            field.selectValue = item;
            field.fieldValue = item;
          } else {
            field.selectedValue[0] = item;
            field.fieldValue[0] = item;
          }
            
          field.datasource.get().then((response: any) => {
            field.updateData(response);
          });
          
          if (keyName === 'CDOPERADOR' || keyName === 'CDFILIAL') {
            const nrapurprec = Metadata.getInstance<any>('NRAPURPREC');
            nrapurprec.datasource.filter[keyName] = item[keyName];
            nrapurprec.disabled = false;
          }
        }
      });
    }
  }
  
  public productMask(value: string) {
    if (value.length === 10) {
      return '#.##.##.###.##';
    }
    
    if (value.length > 5 && value.length <= 8) {
      return '#.##.##.###';
    }
    
    if (value.length === 4 || value.length === 5) {
      return '#.##.##';
    }
    
    if (value.length === 2 || value.length === 3) {
      return '#.##';
    }
    
    return value;
  }
  
  public saveUpdates({ component, event }: any) {
    event?.preventDefault();
    
    Http.post('/updateProducts', component.editedRows).then((response) => {
      if (response.data.dataset.data.success === true) {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'noData',
          text: 'Produtos atualizados com sucesso.',
          color: 'success',
        });
      }
   
      if (response.data.dataset.data.success !== true) {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'noData',
          text: 'Erro ao Atualizar Produtos.',
          color: 'error',
        });
      }
      this.loadGrid();
    });
  }
  
  public cancelUpdates() {
    this.loadGrid();
  }
  
  public changeTab({ event }: any) {
    const tabs: any = {
      0: 'G',
      1: 'C',
    };
    
    this.currentTab = tabs[event];
    this.loadGrid();
  }
  
  public generateAfs() {
    const grid = Metadata.getInstance<ZdGrid>('cmp99000Grid');
    
    const params = {
      rows: grid.selectedRows,
      CDOPERADOR: this.cdOperador,
    };
    
    Http.post('/generateAfs', params).then((response) => {
      if (response.data.dataset.data.erroGeral === 'Z') {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'noData',
          text: 'Sem produtos válidos para geração',
          color: 'error',
        });
      }
      
      if (response.data.dataset.data.erroGeral === 'N') {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'noData',
          text: 'AF Gerado com sucesso',
          color: 'success',
        });
      } else {
        AlertService.show({
          top: true,
          right: true,
          timeout: 2000,
          name: 'noData',
          text: 'Erro ao Gerar AF',
          color: 'error',
        });
      }
      
      this.loadGrid();
    });
  }
}
